import { Component, TemplateRef, ViewChild } from '@angular/core';
import { EditBeneficiaryRegistrationService } from '../edit-beneficiary-registration.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';

@Component({
  selector: 'app-edit-beneficiary-registration',
  templateUrl: './edit-beneficiary-registration.component.html',
  styleUrls: ['./edit-beneficiary-registration.component.css'],
})
export class EditBeneficiaryRegistrationComponent {
  bsModalRef: BsModalRef;

  @ViewChild('updateBeneficiaryRegistration')
  manageagentuserTemplate: TemplateRef<any>;
  @ViewChild('view') view: TemplateRef<any>;

  @ViewChild('viewBankDetail') viewBankDetailModal;
  bsViewBankDetailModalRef: BsModalRef;

  editData: any;
  customerDetails: any;
  customerNo: any;
  allData: any;
  PayoutCurrency: any;
  PhoneCode: any = [];
  PurposeOfTransfer: any = [];
  IdTypeList: any = [];
  SourceOfFund: any = [];
  AccountTypeList: any = [];
  Relationship: any = [];
  CustomerNumber: any;
  bankDetails: any = [];
  bankName: any = '';
  branchName: any = '';
  isFormValid: boolean = false;
  isLoading: boolean = false;

  constructor(
    private modalService: BsModalService,
    private EditBeneficiaryRegistrationSvc: EditBeneficiaryRegistrationService
  ) {}

  ngOnInit(): void {
    this.getRelationship();
    this.getAccountTypeList();
    this.getAccountTypeList();
    this.getSourceOfFund();
    this.getIdTypeList();
    this.getPurposeOfTransfer();
    this.getPhoneCode();
  }

  getCustomer() {
    if (!this.customerNo) {
      alert('Please Enter Customer No.');
      return;
    }
    this.isLoading = true;
    this.EditBeneficiaryRegistrationSvc.getCustomer(this.customerNo).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allData = res;
      },
      error: (err: any) => {
        this.isLoading = false;
        alert(err?.error?.message);
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  onSelectCountryCode(CountryCode) {
    this.getPayoutCurrency(CountryCode);
  }

  getPayoutCurrency(CountryCode, type = 'add') {
    this.EditBeneficiaryRegistrationSvc.getCurrency(CountryCode).subscribe(
      (res) => {
        this.PayoutCurrency = res;
        if (type == 'edit') {
          this.openBeneficiaryModal();
        }
      }
    );
  }

  getPhoneCode() {
    this.EditBeneficiaryRegistrationSvc.getPhoneCode().subscribe((res) => {
      this.PhoneCode = res;
    });
  }

  getPurposeOfTransfer() {
    this.EditBeneficiaryRegistrationSvc.getPurposeOfTransfer().subscribe(
      (res) => {
        this.PurposeOfTransfer = res;
      }
    );
  }

  getIdTypeList() {
    this.EditBeneficiaryRegistrationSvc.getIdType().subscribe((res) => {
      this.IdTypeList = res;
    });
  }

  getSourceOfFund() {
    this.EditBeneficiaryRegistrationSvc.getSourceOfFund().subscribe((res) => {
      this.SourceOfFund = res;
    });
  }

  getAccountTypeList() {
    this.EditBeneficiaryRegistrationSvc.getAccountTypeList().subscribe(
      (res) => {
        this.AccountTypeList = res;
      }
    );
  }

  getRelationship() {
    this.EditBeneficiaryRegistrationSvc.getRelationship().subscribe((res) => {
      this.Relationship = res;
    });
  }

  edit(BeneficiaryNo: any) {
    this.EditBeneficiaryRegistrationSvc.edit(BeneficiaryNo).subscribe((res) => {
      this.editData = res[0];
      if (this.editData?.PayoutCountryCode) {
        this.getPayoutCurrency(this.editData?.PayoutCountryCode, 'edit');
      }
      // this.openBeneficiaryModal();
    });
  }

  viewCustomer(ID) {
    this.EditBeneficiaryRegistrationSvc.viewCustomer(ID).subscribe((res) => {
      const resData = res?.[0]; // Get the first item from the response array
      if (resData) {
        // Extract relevant properties from resData
        const {
          BeneficiaryNo,
          FirstName,
          MiddleName,
          LastName,
          NickName,
          Address,
          DateOfBirth,
          Nationality,
          MobileNumber,
          IDNumber,
          IDType,
          PayOutCountry,
          Currency,
          PaymentMode,
          PaymentBranchCode,
          AccountNumber,
          BankName,
          BranchName,
          BankCountry,
          BankCode,
          IsAuth,
          CustomerNo,
          RelationshipWithCustomer,
        } = resData;

        // Create an object with the extracted properties
        const customerDetailsObj = {
          BeneficiaryNo,
          FirstName,
          MiddleName,
          LastName,
          NickName,
          Address,
          DateOfBirth,
          Nationality,
          MobileNumber,
          IDNumber,
          IDType,
          PayOutCountry,
          Currency,
          PaymentMode,
          PaymentBranchCode,
          AccountNumber,
          BankName,
          BranchName,
          BankCountry,
          BankCode,
          IsAuth,
          CustomerNo,
          RelationshipWithCustomer,
        };

        // Map the object properties to label-value pairs
        this.customerDetails = Object.entries(customerDetailsObj).map(
          ([key, value]) => ({ label: key, value })
        );
      }
    });

    this.bsModalRef = this.modalService.show(this.view);
  }

  openBeneficiaryModal() {
    this.bsModalRef = this.modalService.show(this.manageagentuserTemplate, {
      class: 'modal-lg',
    });
  }

  update(item) {
    if (!item?.BeneficiaryNo) {
      alert('Please Fill Beneficiary No.');
      return;
    }

    if (!item?.PayoutCountryCode) {
      alert('Please Select Payout Country Code.');
      return;
    }

    if (!item?.Currency) {
      alert('Please Select Payout Currency.');
      return;
    }

    if (!item?.PaymentMode) {
      alert('Please Select Payment Mode.');
      return;
    }

    if (!item?.PurposeCode) {
      alert('Please Select Purpose of Transfer.');
      return;
    }

    const selectedPurpose = this.PurposeOfTransfer.find(
      (p) => p.PurposeOfTransferID === Number(item.PurposeCode)
    );

    if (!selectedPurpose) {
      alert('Please Select Purpose of Transfer.');
      return;
    }

    const purposeCode = selectedPurpose.PurposeOfTransferID;
    const purposeValue = selectedPurpose.PurposeOfTranfer;

    if (!item?.BeneficiaryFirstName) {
      alert('Please Fill Beneficiary First Name.');
      return;
    }

    if (!item?.BeneficiaryLastName) {
      alert('Please Fill Beneficiary Last Name.');
      return;
    }

    if (!item?.NickName) {
      alert('Please Fill Beneficiary Nick Name.');
      return;
    }

    if (!item?.BankName) {
      alert('Please Fill Bank Name.');
      return;
    }

    if (!item?.AccountNumber) {
      alert('Please Fill Account Number.');
      return;
    }

    if (!item?.BankBranch) {
      alert('Please Fill Branch Address.');
      return;
    }

    if (!item?.BankCode) {
      alert('Please Fill BankIFSC Code.');
      return;
    }

    const selectedPhoneCode = this.PhoneCode.find(
      (p) => p.NationalityID === Number(item.ReceiverPhoneCountryCode)
    );

    const beneficiaryPhoneCountryCodeText = selectedPhoneCode?.ISOCode
      ? selectedPhoneCode?.ISOCode
      : '';
    const beneficiaryPhoneCountryCodeValue = selectedPhoneCode?.NationalityID
      ? selectedPhoneCode?.NationalityID
      : '';

    const payload = {
      BeneficiaryNo: item?.BeneficiaryNo,
      CustomerNo: item?.CustomerNo,
      FirstName: item?.BeneficiaryFirstName,
      MiddleName: item?.BeneficiaryMiddleName,
      LastName: item?.BeneficiaryLastName,
      NickName: item?.NickName,
      BeneficiaryAddress: item?.BeneficiaryAddress,
      DOB: item?.BeneficiaryDateOfBirth,
      Telephone: item?.BeneficiaryTelephone,
      Nationality: item?.BeneficiaryNationality,
      Idnumber: item?.BeneficiaryIdnumber,
      IdType: Number(item?.BeneficiaryIdType),
      PaymentMode: item?.PaymentMode,
      BankName: item?.BankName,
      DestinationCountry: item?.PayoutCountryCode,
      BankBranch: item?.BranchName,
      BankCode: item?.BankCode,
      BankAccountNo: item?.AccountNumber,
      BankAddress: item?.BankAddress ? item?.BankAddress : '',
      BranchCode: item?.PayOutBranchCode,
      BranchAddress: item?.BankBranch,
      PayoutCurrency: item?.Currency,
      PurposeText: String(purposeValue),
      Purpose: String(purposeCode),
      UserID: getCacheData('G_UserID'),
      CustomerRelation: Number(item?.CustomerRelation),
      BeneficiaryCity: item?.ReceiverCity,
      beneficiaryPhoneCountryCodeValue: beneficiaryPhoneCountryCodeValue,
      beneficiaryPhoneCountryCodeText: beneficiaryPhoneCountryCodeText,
      BeneficiarySourceoffunds: String(item?.ReceiverSourceOfFund)
        ? String(item?.ReceiverSourceOfFund)
        : '0',
      BeneficiaryAccountType: String(item?.ReceiverAccounttype)
        ? String(item?.ReceiverAccounttype)
        : '0',
      BeneficiaryZipCode: item?.ReceiverZipCode,
      Declaration: item?.Declaration ? item?.Declaration : false,
      FatherSpouse: item?.FatherSpouseName,
    };

    this.EditBeneficiaryRegistrationSvc.update(payload).subscribe({
      next: (res: any) => {
        alert('Data updated.');
      },
      error: (err: any) => {
        alert(err?.error?.message);
        this.bsModalRef.hide();
      },
      complete: () => {
        this.bsModalRef.hide();
      },
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  closeModal() {
    if (this.bsModalRef) {
      this.bsModalRef.hide();
    }
  }

  searchBeneficiary() {
    if (!this.editData?.BeneficiaryNo) {
      alert('Please Enter Beneficiary Number.');
      return;
    }
    this.isLoading = true;
    this.EditBeneficiaryRegistrationSvc.edit(
      this.editData?.BeneficiaryNo
    ).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No Data Found.');
          return;
        }
        this.editData = res[0];
        if (this.editData?.PayoutCountryCode) {
          this.getPayoutCurrency(this.editData?.PayoutCountryCode);
        }
      },
      error: (err: any) => {
        this.isLoading = false;
        alert(err?.error?.message);
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  openBankDetailModal(template: TemplateRef<any>) {
    this.bsModalRef.hide();
    this.bankName = '';
    this.branchName = '';
    this.bsViewBankDetailModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  checkFormValidity(): void {
    this.isFormValid = this.bankName.length >= 4 || this.branchName.length >= 4;
  }

  searchBankDetail() {
    if (!this.bankName && !this.branchName) {
      alert('Please enter Bank Name or Branch Name.');
      return;
    }

    const countryCode = this.editData?.PayoutCountryCode || 0;

    this.bankDetails = [];

    this.EditBeneficiaryRegistrationSvc.getBankDetail(
      countryCode,
      this.bankName,
      this.branchName
    ).subscribe({
      next: (res: any) => {
        if (!res || res.length === 0) {
          alert('No data found.');
        } else {
          this.bankDetails = res;
        }
      },
      error: (err: any) => {
        const errorMessage =
          err?.error?.message ||
          'An error occurred while fetching bank details.';
        alert(errorMessage);
      },
      complete: () => {
        this.bankDetails = [];
      },
    });
  }
}
