import { Component, ElementRef, ViewChild } from '@angular/core';
import * as XLSX from 'xlsx';
import { IndianInboundTransactionReportService } from '../services/indian-inbound-transaction-report.service';

@Component({
  selector: 'app-indian-inbound-transaction-report',
  templateUrl: './indian-inbound-transaction-report.component.html',
  styleUrls: ['./indian-inbound-transaction-report.component.css'],
})
export class IndianInboundTransactionReportComponent {
  @ViewChild('indianInboundTransactionTable')
  indianInboundTransactionTable: ElementRef;

  transactionNo: any = '';
  fromDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  
  allIndianInboundTransactions: any = [];
  selectedFileFormat: any = '1';
  isLoading: boolean = false;
  constructor(
    private indianInboundTransactionSvc: IndianInboundTransactionReportService
  ) {}

  ngOnInit(): void {}

  filterByTransactionNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }
    this.isLoading = true;
    this.indianInboundTransactionSvc
      .getDataByTransactionNo(this.transactionNo)
      .subscribe({
        next: (res: any) => {
          this.isLoading = false;
          if (res?.length == 0) {
            this.allIndianInboundTransactions = [];
            alert('No data found.');
            return;
          }

          this.allIndianInboundTransactions = res;
          this.transformData();
        },
        error: (err) => {
          this.isLoading = false; 
          this.allIndianInboundTransactions = [];
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
  }

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }
    this.isLoading = true;
    // Corrected method name
    this.indianInboundTransactionSvc
      .getFillGrid(this.fromDate, this.toDate)
      .subscribe({
        next: (res: any) => {
          this.isLoading = false; 
          if (res?.length == 0) {
            this.allIndianInboundTransactions = [];
            alert('No data found.');
            return;
          }
          this.allIndianInboundTransactions = res;
          this.transformData();
        },
        error: (err) => {
          this.isLoading = false; 
          this.allIndianInboundTransactions = [];
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.indianInboundTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_Indian_Inbound_Transactions_${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  transformData() {
    this.allIndianInboundTransactions = this.allIndianInboundTransactions.map(
      (item) => {
        const newItem = {};
        for (const key in item) {
          if (item.hasOwnProperty(key)) {
            const value =
              item[key] === null || item[key] === undefined ? '' : item[key];
            const newKey = key.replace(/^\*|(?<=,)\*|\*(?=,|$)|\s+/g, ''); // Remove asterisk and replace spaces with underscores
            newItem[newKey] = value;
          }
        }
        return newItem;
      }
    );
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
