import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class RemittancePaidUnpaidService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getCountries(countryType: string): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/Remittance/remittance-paid-unpaid/country?CountryType=${countryType}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgents(countryId: number): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/Remittance/remittance-paid-unpaid/agent?CountryID=${countryId}&AgentID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getStatus(): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/Remittance/remittance-paid-unpaid/status`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getTransferType(): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/Remittance/remittance-paid-unpaid/transfer-type`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getFillGrid(payload: any = {}): Observable<any> {
    return this.http.post(
      `${this.url}/api/master/reports/Remittance/remittance-paid-unpaid/grid`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
