import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class SendingAgentMarginService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getSendCountryList() {
    return this.http.get(
      ` ${environment.apiUrl}/api/master/sa-margin/send-country`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentList(sendCountryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/send-agent?SendCountryID=${sendCountryId}&SessionG_Agent_ID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getFillGrid(sendCountryId: any = 0, sendAgentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/fill-grid?SendCountryID=${sendCountryId}&SendAgentID=${sendAgentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendCountryAddList() {
    return this.http.get(
      ` ${environment.apiUrl}/api/master/sa-margin/add/send-country?AsPerPermission=true&permissionType=2`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendAgentAddList(sendCountryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/add/send-agent?CountryID=${sendCountryId}&permission=2`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentCurrency(sendCountryId: any = 0, sendAgentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/add/agent-currency?AgentID=${sendAgentId}&CountryID=${sendCountryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getReceiveCountry(sendCountryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/add/receive-country?PayInCountryId=${sendCountryId}&permission=1`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getReceiveCurrency(sendCountryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/sa-margin/add/receive-currency?CountryID=${sendCountryId}&AgentID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  updateSendingAgentMargin(payload: any) {
    return this.http.patch(
      `${environment.apiUrl}/api/master/sa-margin/update`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  addMargin(params: {
    PayInCountryId: number;
    Agent_ID: number;
    Agent_CurrencyID: number;
    Target_CountryID: number;
    Target_CurrencyID: number;
    Margin_Rate: any;
    UserID: string;
  }) {
    const headers = {
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    };

    const httpParams = new HttpParams()
      .set('PayInCountryId', params.PayInCountryId.toString())
      .set('Agent_ID', params.Agent_ID.toString())
      .set('Agent_CurrencyID', params.Agent_CurrencyID.toString())
      .set('Target_CountryID', params.Target_CountryID.toString())
      .set('Target_CurrencyID', params.Target_CurrencyID.toString())
      .set('Margin_Rate', params.Margin_Rate.toString())
      .set('UserID', params.UserID.toString());

    const url = `${
      environment.apiUrl
    }/api/master/sa-margin/add?PayInCountryId=${params.PayInCountryId.toString()}&Agent_ID=${params.Agent_ID.toString()}&Agent_CurrencyID=${params.Agent_CurrencyID.toString()}&Target_CountryID=${params.Target_CountryID.toString()}&Target_CurrencyID=${params.Target_CurrencyID.toString()}&Margin_Rate=${params.Margin_Rate.toString()}&UserID=${params.UserID.toString()}`;

    return this.http.post(url, {}, { headers });
  }
}
