import { Component, OnInit } from '@angular/core';
import { TransactionsStatusReportService } from '../services/transactions-status-report.service';

@Component({
  selector: 'app-transactions-status-report',
  templateUrl: './transactions-status-report.component.html',
  styleUrls: ['./transactions-status-report.component.css'],
})
export class TransactionsStatusReportComponent implements OnInit {
  agentAccounts: any[] = [];
  branches: any[] = [];
  ccys: any[] = [];
  states: any[] = [];
  selectedAgentAccount: any;
  selectedBranch: any;
  selectedCCY: any;
  selectedState: any;
  startDate: string;
  endDate: string;
  sessionGAgentId: any;

  constructor(private transactionService: TransactionsStatusReportService) {
    const currentDate = new Date().toISOString().substring(0, 10);
    this.startDate = currentDate;
    this.endDate = currentDate;
  }

  ngOnInit(): void {
    this.loadAgentAccounts();
  }

  loadAgentAccounts() {
    const agentId =
      this.sessionGAgentId !== undefined ? this.sessionGAgentId : 0;
    this.transactionService
      .getTransactionStatusPaidUnpaidReportByAgentAccount(agentId)
      .subscribe((data: any) => {
        this.agentAccounts = data;
        this.onAgentAccountChange(0);
      });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  onAgentAccountChange(selectedAgentAccountId: any) {
    if (selectedAgentAccountId !== undefined) {
      this.selectedAgentAccount = selectedAgentAccountId;

      this.transactionService
        .getTransactionStatusPaidUnpaidReportByBranch(selectedAgentAccountId)
        .subscribe((data: any) => {
          this.branches = data;
        });

      this.transactionService
        .getTransactionStatusPaidUnpaidReportByCCY(selectedAgentAccountId)
        .subscribe((data: any) => {
          this.ccys = data;
        });

      this.transactionService
        .getTransactionStatusReport()
        .subscribe((data: any) => {
          this.states = data;
        });
    }
  }

  onSubmit() {}
}
