import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TurnOverReportService } from '../services/turn-over-reports.service';

@Component({
  selector: 'app-turn-over-report',
  templateUrl: './turn-over-report.component.html',
  styleUrls: ['./turn-over-report.component.css'],
})
export class TurnOverReportComponent implements OnInit {
  countries: any[] = [];
  showTable: boolean = false;
  reportData: any[] = [];
  grandTotal: any = {};
  reportForm: FormGroup;
  isLoading: boolean = false;

  constructor(
    private fb: FormBuilder,
    private turnOverReportService: TurnOverReportService
  ) {}

  ngOnInit(): void {
    this.createForm();
    this.getCountries();
  }

  createForm(): void {
    const currentStartDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentEndDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    this.reportForm = this.fb.group({
      country: ['', Validators.required],
      fromDate: [currentStartDate, Validators.required],
      toDate: [currentEndDate, Validators.required],
    });
  }

  getCountries(): void {
    this.turnOverReportService.getCountries().subscribe(
      (data: any[]) => {
        this.countries = data;
        if (this.countries.length > 0) {
          this.reportForm.patchValue({ country: this.countries[0]?.CountryId });
        }
      },
      (error: any) => {
        console.error('Error fetching countries:', error);
      }
    );
  }

  generateReport(): void {
    this.isLoading = true;
    if (this.reportForm.valid) {
      const { country, fromDate, toDate } = this.reportForm.value;
      this.turnOverReportService
        .getCountryWiseSummaryReport(country, fromDate, toDate)
        .subscribe(
          (data: any[]) => {
            this.reportData = data;
            this.showTable = true;
            this.isLoading = false;
          },
          (error: any) => {
            console.error('Error fetching country-wise summary report:', error);
            this.isLoading = false;
          }
        );
    } else {
      console.error('Please fill all required fields.');
      this.reportForm.markAllAsTouched();
      this.isLoading = false;
    }
  }

  get totalPrincipalUSD() {
    let total = 0;
    for (let index = 0; index < this.reportData.length; index++) {
      const element = this.reportData[index];
      total = total + element.PrincipalUSD;
    }
    return total;
  }

  get totalNoofTrxns() {
    let total = 0;
    for (let index = 0; index < this.reportData.length; index++) {
      const element = this.reportData[index];
      total = total + element.NoOfTrans;
    }
    return total;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
