import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { ViewCancelTransactionService } from '../services/view-cancel-transaction.service';
import { ViewReceiveTransactionService } from '../services/view-receive-transaction.service';
import { AuthService } from '../services/auth.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import * as XLSX from 'xlsx';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-view-receive-transaction',
  templateUrl: './view-receive-transaction.component.html',
  styleUrls: ['./view-receive-transaction.component.css'],
})
export class ViewReceiveTransactionComponent {
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;
  @ViewChild('receiveTransactionTable') receiveTransactionTable: ElementRef;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 'ALL';
  statusList: any = [];
  selectedStatusList: any = '-1';
  transactionNo: any = '';
  agentId: any;
  startDate: any;
  endDate: any;
  agentBranchID: any = 'ALL';

  allReceiveTransactions: any = [];
  selectedFileFormat: any = '1';
  receiptData: any = {};

  constructor(
    private viewCancelTransactionSvc: ViewCancelTransactionService,
    private viewReceiveTransactionSvc: ViewReceiveTransactionService,
    private authSvc: AuthService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) { }

  ngOnInit(): void {
    this.getCountryList();
    this.getStatusList();
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  getCountryList() {
    this.viewReceiveTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryId;
        this.getAgentList(res?.[0]?.CountryId);
      },
      error: () => {
        this.countryList = [];
      },
      complete: () => { },
    });
  }

  ReceiveTrnDetails() {
    if (!this.startDate || !this.endDate) {
      alert('Please Select From & To Date!');
      return;
    }
    if (!this.selectedCountryList) {
      alert('Please Select Country!');
      return;
    }
    if (!this.selectedAgentList) {
      alert('Please Select Agent!');
      return;
    }
    const payload = {
      FromDate: this.startDate,
      ToDate: this.endDate,
      AgentID: this.selectedAgentList,
      StatusID: this.selectedStatusList,
      AgentBranchID: this.selectedAgentBranchList,
    };
    this.viewReceiveTransactionSvc.ReceiveTrnDetails(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allReceiveTransactions = res;
      },
      error: (err: any) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => { },
    });
  }

  ReceiveTrnNoDetails() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No.');
      return;
    }
    const payload = {
      G_UserGroup: getCacheData('G_UserGroup'),
      SubUserGroup_Id: getCacheData('SubUserGroup_Id'),
      IsHeadOffice: 'true',
      TransactionNumber: this.transactionNo,
      AgentID: 0,
      AgentBranchID: '',
    };
    this.viewReceiveTransactionSvc.ReceiveTrnNoDetails(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allReceiveTransactions = res;
      },
      error: (err: any) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => { },
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.viewReceiveTransactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgentList = res?.[0]?.Agent_ID;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {
        this.getAgentBranchList(this.selectedAgentList);
      },
    });
  }

  onAgentChange(agentId: any) {
    this.agentId = agentId;
  }

  getAgentBranchList(agentId: any = 0) {
    this.viewReceiveTransactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
        // this.selectedAgentBranchList = res?.[0]?.AgentBranch_ID;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => { },
    });
  }

  onSelectAgentBranch(agentBranchID) {
    this.agentBranchID = agentBranchID;
  }

  getStatusList() {
    this.viewReceiveTransactionSvc.getStatusList().subscribe({
      next: (res: any) => {
        this.statusList = res;
        // this.selectedStatusList = res?.[0]?.TrStatusID;
      },
      error: () => {
        this.statusList = [];
      },
      complete: () => { },
    });
  }

  get totalPayOutAmount() {
    let total = 0;
    for (let index = 0; index < this.allReceiveTransactions.length; index++) {
      const element = this.allReceiveTransactions[index];
      if (element?.LC_Amount) {
        // Directly add LC_Amount to total
        total += Number(element.LC_Amount);
      }
    }
    return total;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.SRTRNo;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.receiveTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Cancellation_Checker${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }
}
