import {
  ChangeDetectorRef,
  Component,
  ElementRef,
  OnInit,
  QueryList,
  ViewChild,
  ViewChildren,
} from '@angular/core';
import { VostroAccountBalanceService } from '../vostro-account-balance.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-vostro-account-balance',
  templateUrl: './vostro-account-balance.component.html',
  styleUrls: ['./vostro-account-balance.component.css'],
})
export class VostroAccountBalanceComponent implements OnInit {
  @ViewChildren('tableRow') tableRows!: QueryList<ElementRef>;

  vostroBalanceData: any[] = [];
  dropDownCountrys: any = [];
  dropDownAgentList: any = [];
  dropownCurrencyList: any = [];
  dropDownAccInfoList: any = [];
  isLoading: boolean = false;

  constructor(
    private vostroBalanceSvc: VostroAccountBalanceService,
    private cdr: ChangeDetectorRef,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit() {
    // this.getCurrency();
    this.getCountry();
    // this.getAgent();
    this.getVostroBalance();
  }

  getVostroBalance() {
    this.isLoading = true;
    this.vostroBalanceSvc.getVostroBalance().subscribe({
      next: (res: any) => {
        this.vostroBalanceData = res.map((item) => ({
          ...item,
          selected: false,
          newRow: false,
          // disable: !this.isNewRow(item), // Set disable status based on row status
          agents: [],
          currencies: [],
        }));
        this.populateAgentsAndCurrenciesForAllRows();
        this.isLoading = false;
      },
      error: (err: any) => {
        console.error(err);
        this.isLoading = false;
      },
    });
  }

  getCountry() {
    this.vostroBalanceSvc.getCountry().subscribe({
      next: (res: any) => {
        this.dropDownCountrys = res;
      },
      error: (err: any) => {
        console.error(err);
      },
    });
  }

  onCountryChange(countryId: number, item: any, index: number) {
    if (countryId) {
      this.getAgent(countryId, index, item);
      this.getCurrency(countryId, index, item);
    }
  }

  populateAgentsAndCurrenciesForAllRows() {
    this.vostroBalanceData.forEach((item, index) => {
      if (item.Country) {
        this.getAgent(item.Country, index);
        this.getCurrency(item.Country, index);
      }
    });
  }

  getAgent(countryId: number, index: number, item: any = '') {
    this.vostroBalanceSvc.getAgent(countryId).subscribe({
      next: (res: any) => {
        this.vostroBalanceData[index].agents = res;
        // Update item.Agent with the first agent's ID if available
        if (res.length > 0 && item) {
          // item.Agent = res?.[0]?.Agent_ID;
          item.Agent = -1;
        }
        if (countryId && item?.Currency && item?.Agent != -1) {
          this.getAccInfo(
            countryId,
            Number(item.Agent),
            Number(item?.Currency),
            index
          );
        }
      },
      error: (err: any) => {
        console.error(err);
      },
    });
  }

  onAgentChange(agentId: number, item: any, index: number) {
    if (agentId) {
      this.getCurrency(Number(item?.Country), index, item);
    }
  }

  getCurrency(countryId: number, index: number, item: any = '') {
    this.vostroBalanceSvc.getCurrency(countryId).subscribe({
      next: (res: any) => {
        this.vostroBalanceData[index].currencies = res;
        // Update item.Currency with the first currency's ID if available
        if (res.length > 0 && item) {
          // item.Currency = res?.[0]?.Currency_Id;
          item.Currency = -1;
        }
        if (countryId && item?.Currency && item.Currency != -1) {
          this.getAccInfo(
            countryId,
            Number(item?.Agent),
            Number(item?.Currency),
            index
          );
        }
      },
      error: (err: any) => {
        console.error(err);
      },
    });
  }

  getAccInfo(countryId = 0, agentId = 0, currencyId = 0, index: number) {
    this.vostroBalanceSvc.getAccInfo(countryId, agentId, currencyId).subscribe({
      next: (res: any) => {
        this.dropownCurrencyList = res;
        this.vostroBalanceData[index].AccNum = res?.[0]?.AccNum;
        // Trigger change detection
        this.cdr.detectChanges();
      },
      error: (err: any) => {
        console.error(err);
      },
    });
  }

  updateVostroBal() {
    const checkedItems = this.vostroBalanceData.filter(
      (obj) => obj.newRow == true && obj.DepositAmt == '0'
    );

    if (checkedItems.length > 0) {
      alert('Manage Balance needs to be greater than 0.');
      return;
    }

    // Filter out items with null or '0' DepositAmt
    let payload = this.vostroBalanceData.filter(
      (obj) => obj.DepositAmt !== null && obj.DepositAmt !== '0'
    );

    if (payload.length == 0) {
      alert('Please add items for submit.');
      return;
    }

    const agent = payload.find((a) => a.Agent == -1);
    if (agent) {
      alert('Please Select Agent.');
      return;
    }

    const currency = payload.find((c) => c.Currency == -1);
    if (currency) {
      alert('Please Select Currency.');
      return;
    }

    this.isLoading = true;

    // Key mapping
    const keyMapping = {
      AccBal: 'AccBal',
      AccNum: 'AccNum',
      Agent: 'AgentID',
      Country: 'CountryID',
      Currency: 'CurrencyID',
      DepositAmt: 'DepositAmt',
      // IsOldRec: 'IsOldRec',
      PendingTxnAmt: 'PendingTxnAmt',
      TxnType: 'TxnType',
    };

    // Map payload to new structure and remove unwanted fields
    payload = payload.map((obj) => {
      const newObj: any = {};
      for (const key in obj) {
        if (keyMapping[key]) {
          if (
            keyMapping[key] === 'CountryID' ||
            keyMapping[key] === 'AgentID' ||
            keyMapping[key] === 'CurrencyID' ||
            keyMapping[key] === 'DepositAmt'
          ) {
            newObj[keyMapping[key]] = Number(obj[key]);
          } else {
            newObj[keyMapping[key]] = obj[key];
          }
        }
      }
      newObj['G_UserName'] = getCacheData('username'); // Ensure G_UserName is set to '0'
      newObj['IsOldRec'] = obj.IsOldRec === '1' ? '1' : '2';
      return newObj;
    });

    // API call
    this.vostroBalanceSvc.updateVostroBalance(payload).subscribe({
      next: (res: any) => {
        alert(res);
        this.isLoading = false;
        this.getVostroBalance();
      },
      error: (err: any) => {
        this.isLoading = false;
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
    });
  }

  addRow() {
    const newRow = {
      Country: this.dropDownCountrys?.[0]?.CountryId || '',
      Agent: '-1',
      Currency: '-1',
      AccNum: '',
      AccBal: 0,
      PendingTxnAmt: 0,
      TxnType: '1',
      DepositAmt: null,
      newRow: true,
      selected: false,
      disable: false, // New rows are always editable
      agents: [], // Initialize empty array for agents
      currencies: [], // Initialize empty array for currencies
    };
    this.vostroBalanceData.push(newRow);

    // Fetch agents and currencies for the newly added row
    const newRowIndex = this.vostroBalanceData.length - 1;
    if (newRow.Country) {
      this.getAgent(newRow.Country, newRowIndex);
      this.getCurrency(newRow.Country, newRowIndex);
    }

    // Scroll to the newly added row
    setTimeout(() => {
      const newRowElement = this.tableRows.toArray()[newRowIndex];
      newRowElement.nativeElement.scrollIntoView({
        behavior: 'smooth',
        block: 'start',
      });
    }, 100); // Adding a slight delay to ensure the DOM is updated
  }

  onCurrencyChange(currencyId, item, index) {
    if (currencyId && item) {
      this.getAccInfo(
        Number(item.Country),
        Number(item.Agent),
        currencyId,
        index
      );
    }
  }

  deleteSelectedRows() {
    this.vostroBalanceData = this.vostroBalanceData.filter(
      (item) => !item.selected
    );
  }

  selectAll(event: any) {
    const isChecked = event.target.checked;
    this.vostroBalanceData.forEach((item) => (item.selected = isChecked));
  }

  // Function to determine if a row is new
  isNewRow(item: any): boolean {
    // Add your logic to determine if a row is new
    // Example: return true if AccNum is empty, false otherwise
    return !item.AccNum;
  }

  // Function to toggle the disable status based on checkbox state
  toggleDisable(item: any) {
    item.disable = !item.selected;
  }

  // Function to determine if the current item should be disabled
  shouldDisable(item: any): boolean {
    // Disable if the row is not new and checkbox is not selected
    return !item.newRow;
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
